@@ -46,9 +46,9 @@ def mol2(filename):
                     current_residue = Residue(ResidueType.types[words[7]])
                 if temp:
                     current_residue.type.Add_Atom(words[1], AtomType.types[words[5]], *words[2:5])
-                    current_residue.type.atoms[-1].update(**{"charge[e]": float(words[8])})
+                    current_residue.type.atoms[-1].Update(**{"charge[e]": float(words[8])})
                 current_residue.Add_Atom(words[1], AtomType.types[words[5]], *words[2:5])
-                current_residue.atoms[-1].update(**{"charge[e]": float(words[8])})
+                current_residue.atoms[-1].Update(**{"charge[e]": float(words[8])})
                 atom_residue_map[words[0]]=[words[1], current_residue, current_residue_index, temp, current_residue.atoms[-1], current_residue.type.atoms[-1]]
             elif flag == "BOND":
                 words = line.split()
@@ -82,12 +82,13 @@ def pdb(filename, judge_HIS = True):
     residue_type_map = []
     current_residue_count = -1
     current_residue_index = None
+    current_resname = None
     current_HIS = {"DeltaH":False, "EpsilonH":False}
     with open(filename) as f:
         for line in f:
             if line.startswith("ATOM") or line.startswith("HETATM"):
                 resindex = int(line[22:26])
-                resname = line[17:20]
+                resname = line[17:20].strip()
                 atomname = line[12:16].strip()
                 if current_residue_index == None:
                     current_residue_count += 1
@@ -104,8 +105,9 @@ def pdb(filename, judge_HIS = True):
                         resname = GlobalSetting.PDBResidueNameMap["head"][resname]
                     residue_type_map.append(resname)
                     current_residue_index = resindex
+                    current_resname = resname
                     chain[chr(ord("A") + len(chain.keys()))] = {resindex:current_residue_count}
-                elif current_residue_index != resindex:
+                elif current_residue_index != resindex or current_resname != resname:
                     if judge_HIS and residue_type_map and residue_type_map[-1] in GlobalSetting.HISMap["HIS"].keys():
                         if current_HIS["DeltaH"]: 
                             if current_HIS["EpsilonH"]:
@@ -118,6 +120,7 @@ def pdb(filename, judge_HIS = True):
                     current_residue_count += 1
                     residue_type_map.append(resname)
                     current_residue_index = resindex
+                    current_resname = resname
                     chain[chr(ord("A") + len(chain.keys()) - 1)][resindex] = current_residue_count
                 if judge_HIS and resname in GlobalSetting.HISMap["HIS"].keys():
                     if atomname == GlobalSetting.HISMap["DeltaH"]:
@@ -126,6 +129,7 @@ def pdb(filename, judge_HIS = True):
                         current_HIS["EpsilonH"] = True
             elif line.startswith("TER"):
                 current_residue_index = None
+                current_resname = None
                 if residue_type_map[-1] in GlobalSetting.PDBResidueNameMap["tail"].keys():
                    residue_type_map[-1] = GlobalSetting.PDBResidueNameMap["tail"][residue_type_map[-1]]
                 if judge_HIS and residue_type_map and residue_type_map[-1] in GlobalSetting.HISMap["HIS"].keys():
@@ -156,26 +160,31 @@ def pdb(filename, judge_HIS = True):
 
     current_residue_count = -1
     current_residue = None
+    current_resname = None
     links = []
     with open(filename) as f:
         for line in f:
             if line.startswith("ATOM") or line.startswith("HETATM"):
                 resindex = int(line[22:26])
+                resname = line[17:20].strip()
                 atomname = line[12:16].strip()
                 x = float(line[30:38])
                 y = float(line[38:46])
                 z = float(line[46:54])
-                if not current_residue_index or current_residue_index != resindex:
+                if current_residue_index is None or current_residue_index != resindex or \
+                   current_resname != resname:
                     current_residue_count += 1
                     if current_residue:
                         molecule.Add_Residue(current_residue)
-                        if current_residue.type.tail and ResidueType.types[residue_type_map[current_residue_count]].head:
-                            links.append(current_residue_count)
+                        if current_residue_index is not None and current_residue.type.tail and ResidueType.types[residue_type_map[current_residue_count]].head:
+                            links.append(len(molecule.residues))
                     current_residue = Residue(ResidueType.types[residue_type_map[current_residue_count]])
                     current_residue_index = resindex
+                    current_resname = resname
                 current_residue.Add_Atom(atomname, x = x, y = y, z = z)
             elif line.startswith("TER"):
                 current_residue_index = None
+                current_resname = None
                 
     if current_residue:
         molecule.Add_Residue(current_residue)
@@ -505,16 +514,16 @@ def ffitp(filename, macros = {}):
             words = line.split()
             func = words[2]
             if func == "1":
-                output["bonds"] += "{atom1}-{atom2} {b} {k}\n".format(atom1 = words[0], atom2 = words[1], b = float(words[3]), k = float(words[4]))
+                output["bonds"] += "{atom1}-{atom2} {b} {k}\n".format(atom1 = words[0], atom2 = words[1], b = float(words[3]), k = float(words[4])/2)
             else:
                 raise NotImplementedError
         elif iterator.flag == "angletypes":
             words = line.split()
             func = words[3]
             if func == "1":
-                output["angles"] += "-".join(words[:3]) + " " + " ".join(words[4:]) + "\n"
+                output["angles"] += "-".join(words[:3]) + " {b} {k}".format(b = float(words[4]), k = float(words[5])/2)  + "\n"
             elif func == "5":
-                output["Urey-Bradley"] += "-".join(words[:3]) + " " + " ".join(words[4:])  + "\n"
+                output["Urey-Bradley"] += "-".join(words[:3]) + " {b} {k} {b2} {k2}".format(b = float(words[4]), k = float(words[5])/2, b2 = float(words[6]), k2 = float(words[7])/2)  + "\n"
             else:
                 raise NotImplementedError
         elif iterator.flag == "dihedraltypes":
@@ -523,14 +532,23 @@ def ffitp(filename, macros = {}):
             if func == "1":
                 output["dihedrals"] += "-".join(words[:4]) + " " + " ".join(words[5:]) + " 0\n"
             elif func == "2":
-                output["impropers"] += "-".join([words[1], words[2], words[0], words[3]]) + " " + " ".join(words[5:]) + "\n"
+                temp1 = [words[1], words[2], words[0], words[3]]
+                temp2 = [words[1], words[2], words[3], words[0]]
+                if words[0][0].upper() in ("C", "N", "S"):
+                    output["impropers"] += "-".join(temp1) + " {b} {k}".format(b = float(words[5]), k = float(words[6])/2)  + "\n"
+                else:
+                    output["impropers"] += "-".join(temp2) + " {b} {k}".format(b = float(words[5]), k = float(words[6])/2)  + "\n"
             elif func == "9":
                 for i in range(5,len(words),20):
                     output["dihedrals"] += "-".join(words[:4]) + " " + " ".join(words[i:i+3]) + " 0\n"
+            else:
+                raise NotImplementedError
         elif iterator.flag == "cmaptypes":
             words = line.split()
-            output["cmaps"]["-".join(words[:5])] = {"resolution": int(words[7]), "parameters":list(map(float, words[8:]))}
+            output["cmaps"]["-".join(words[:5])] = {"resolution": int(words[7]), "parameters":list(map(lambda x:float(x) / 4.184, words[8:]))}
         elif iterator.flag == "nonbond_params":
             words = line.split()
             output["LJ"] += "{type1}-{type2} {V} {W}\n".format(type1=words[0], type2=words[1], V=float(words[3]), W=float(words[4]))
-    return output
\ No newline at end of file
+    return output
+    
+sys.modules['__main__'].__dict__["loadffitp"] = ffitp
